/* simple bytea and base64 encoders/decoders              */
/* (C) 2001 by Joerg Hessdoerfer                          */
/* Questions and comments: Joerg.Hessdoerfer@sea-gmbh.com */

#ifndef __ENCODERS_H__
#define __ENCODERS_H__

/* return the size of bytea encoded memory block */
size_t bytea_encoded_size(const unsigned char *src, size_t size);

/* return the size of decoded bytea memory block */
size_t bytea_decoded_size(const unsigned char *src, size_t size);

/* this routine REQUIRES a pre-allocated memory block for encoding */
/* result. Please use bytea_encoded_size() to calculate the required */
/* mem size */
void encode_bytea_prealloced(const unsigned char *src, unsigned char *dst, size_t size);

/* this routine REQUIRES a pre-allocated memory block for encoding */
/* result. Please use bytea_encoded_size() to calculate the required */
/* mem size */
void decode_bytea_prealloced(const unsigned char *src, unsigned char *dst, size_t size);

/* this routine returns a pointer to a block of bytea-encoded memory, with a trailing 0 */
/* returned ptr must be free()ed */
unsigned char *encode_bytea(const unsigned char *src, size_t size, size_t *used);

/* this routine returns a pointer to a block of memory decoded from bytea input */
/* warning: no trailing 0! */
/* returned ptr must be free()ed */
unsigned char *decode_bytea(const unsigned char *src, size_t size, size_t *used);

/* return the size of base64 encoded memory block */
size_t base64_encoded_size(const unsigned char *src, size_t size, int with_lf);

/* return the size of decoded base64 memory block */
size_t base64_decoded_size(const unsigned char *src, size_t size);

/* this routine REQUIRES a pre-allocated memory block for encoding */
/* result. Please use base64_encoded_size() to calculate the required */
/* mem size */
void encode_base64_prealloced(const unsigned char *src, unsigned char *dst, size_t size, int with_lf);

/* this routine REQUIRES a pre-allocated memory block for encoding */
/* result. Please use base64_encoded_size() to calculate the required */
/* mem size */
void decode_base64_prealloced(const unsigned char *src, unsigned char *dst, size_t size);

/* this routine returns a pointer to a block of base64-encoded memory, with a trailing 0 */
/* returned ptr must be free()ed */
unsigned char *encode_base64(const unsigned char *src, size_t size, size_t *used, int with_lf);

/* this routine returns a pointer to a block of memory decoded from base64 input */
/* warning: no trailing 0! */
/* returned ptr must be free()ed */
unsigned char *decode_base64(const unsigned char *src, size_t size, size_t *used);

#endif



