#include <stdio.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/stat.h>
#include <unistd.h>
#include "encoders.h"

int main(int argc, char **argv)
{
  char *input, *output, *decoded;
  size_t enc_size, dec_size, sz;
  struct stat st_b;
  int file,i;
  
  if(argc != 2)
    {
      fprintf(stderr, "Usage: test file_to_encode\n");
      exit(1);
    }

  if(stat(argv[1], &st_b))
    {
      fprintf(stderr, "test: file '%s' not found\n", argv[1]);
      exit(1);
    }

  file = open(argv[1], O_RDONLY);

  if(file < 0)
    {
      fprintf(stderr, "test: can't open '%s'\n", argv[1]);
      exit(1);
    }

  input = (char *)malloc(st_b.st_size);

  if(input == NULL)
    {
      fprintf(stderr, "test: can't read '%s': not enough memory\n", argv[1]);
      exit(1);
    }

  if(read(file, input, st_b.st_size) != st_b.st_size)
    {
      fprintf(stderr, "test: error read()ing '%s'\n", argv[1]);
      exit(1);
    }

  close(file);
        
  printf("bytea:\n");

  
  output = encode_bytea(input, st_b.st_size, &enc_size);
  if(output == NULL)
    {
      fprintf(stderr, "test: encode returned NULL ptr\n");
      exit(1);
    }
  printf("encoded size: %d\n", enc_size);
  
      
  decoded = decode_bytea(output, enc_size, &dec_size);

  printf("decoded size: %d\n", dec_size);

  if(dec_size != st_b.st_size)
    {
      fprintf(stderr, "test: size mismatch\n");
      exit(1);
    }

  if(decoded == NULL)
    {
      fprintf(stderr, "test: decode returned NULL ptr\n");
      exit(1);
    }
      
  for(i = 0; i < st_b.st_size; i++)
    if(decoded[i] != input[i]) printf("Mismatch (pos: %04d): %02x => %02x\n", i, input[i], decoded[i]);

  free(output);
  free(decoded);
  
  printf("base64: (no LF)\n");

  output = encode_base64(input, st_b.st_size, &enc_size, 0);
  
  printf("encoded size: %d\n", enc_size);
  
  if(output == NULL)
    {
      fprintf(stderr, "test: encode returned NULL ptr\n");
      exit(1);
    }
      
  decoded = decode_base64(output, enc_size, &dec_size);

  printf("decoded size: %d\n", dec_size);
  if(dec_size != st_b.st_size)
    {
      fprintf(stderr, "test: size mismatch\n");
      exit(1);
    }
  
  
  for(i = 0; i < st_b.st_size; i++)
    if(decoded[i] != input[i]) printf("Mismatch (pos: %04d): %02x => %02x\n", i, input[i], decoded[i]);

  free(output);
  free(decoded);

  printf("base64: (with LF)\n");

  output = encode_base64(input, st_b.st_size, &enc_size, 1);
  
  if(output == NULL)
    {
      fprintf(stderr, "test: encode returned NULL ptr\n");
      exit(1);
    }
  printf("encoded size: %d\n", enc_size);
  
  decoded = decode_base64(output, enc_size, &dec_size);
  
  printf("decoded size: %d\n", dec_size);
  if(dec_size != st_b.st_size)
    {
      fprintf(stderr, "test: size mismatch\n");
      exit(1);
    }
  
  for(i = 0; i < st_b.st_size; i++)
    if(decoded[i] != input[i]) printf("Mismatch (pos: %04d): %02x => %02x\n", i, input[i], decoded[i]);

  free(output);
  free(decoded);
  free(input);

  exit(0);
}

